@@ -300,7 +300,7 @@ static void mimetype_verify(void){
   int i;
   for(i=1; i<count(aMime); i++){
     if( fossil_strcmp(aMime[i-1].zSuffix,aMime[i].zSuffix)>=0 ){
-      fossil_fatal("mimetypes out of sequence: %s before %s",
+      fossil_panic("mimetypes out of sequence: %s before %s",
                    aMime[i-1].zSuffix, aMime[i].zSuffix);
     }
   }
@@ -469,6 +469,10 @@ int doc_is_embedded_html(Blob *pContent, Blob *pTitle){
 int doc_load_content(int vid, const char *zName, Blob *pContent){
   int writable = db_is_writeable("repository");
   int rid;   /* The RID of the file being loaded */
+  if( writable ){
+    db_end_transaction(0);
+    db_begin_write();
+  }
   if( !db_table_exists("repository", "vcache") || !writable ){
     db_multi_exec(
       "CREATE %s TABLE IF NOT EXISTS vcache(\n"
@@ -641,11 +645,19 @@ void doc_page(void){
       }
     }
     if( isUV ){
-      if( db_table_exists("repository","unversioned")
-       && unversioned_content(zName, &filebody)==0
-      ){
-        rid = 1;
-        zDfltTitle = zName;
+      if( db_table_exists("repository","unversioned") ){
+        Stmt q;
+        db_prepare(&q, "SELECT hash, mtime FROM unversioned"
+                       " WHERE name=%Q", zName);
+        if( db_step(&q)==SQLITE_ROW ){
+          etag_check(ETAG_HASH, db_column_text(&q,0));
+          etag_last_modified(db_column_int64(&q,1));
+        }
+        db_finalize(&q);
+        if( unversioned_content(zName, &filebody)==0 ){
+          rid = 1;
+          zDfltTitle = zName;
+        }
       }
     }else if( fossil_strcmp(zCheckin,"ckout")==0 ){
       /* Read from the local checkout */
@@ -658,8 +670,8 @@ void doc_page(void){
       }
       fossil_free(zFullpath);
     }else{
-      vid = name_to_typed_rid(zCheckin, "ci");
-      rid = doc_load_content(vid, zName, &filebody);
+      vid = symbolic_name_to_rid(zCheckin, "ci");
+      rid = vid>0 ? doc_load_content(vid, zName, &filebody) : 0;
     }
   }
   g.zPath = mprintf("%s/%s", g.zPath, zPathSuffix);
@@ -758,7 +770,6 @@ doc_not_found:
     @ in %z(href("%R/tree?ci=%T",zCheckin))%h(zCheckin)</a>
   }
   style_footer();
-  db_end_transaction(0);
   return;
 }
 
@@ -841,6 +852,7 @@ void logo_page(void){
   Blob logo;
   char *zMime;
 
+  etag_check(ETAG_CONFIG, 0);
   zMime = db_get("logo-mimetype", "image/gif");
   blob_zero(&logo);
   db_blob(&logo, "SELECT value FROM config WHERE name='logo-image'");
@@ -849,7 +861,6 @@ void logo_page(void){
   }
   cgi_set_content_type(zMime);
   cgi_set_content(&logo);
-  g.isConst = 1;
 }
 
 /*
@@ -875,6 +886,7 @@ void background_page(void){
   Blob bgimg;
   char *zMime;
 
+  etag_check(ETAG_CONFIG, 0);
   zMime = db_get("background-mimetype", "image/gif");
   blob_zero(&bgimg);
   db_blob(&bgimg, "SELECT value FROM config WHERE name='background-image'");
@@ -883,7 +895,6 @@ void background_page(void){
   }
   cgi_set_content_type(zMime);
   cgi_set_content(&bgimg);
-  g.isConst = 1;
 }
 
 
