@@ -716,11 +716,11 @@ void ci_page(void){
           zPJ[jj] = '_';
         }
       }
-      zUrl = mprintf("%R/tarball/%t-%S.tar.gz?r=%s", zPJ, zUuid, zUuid);
+      zUrl = mprintf("%R/tarball/%S/%t-%S.tar.gz", zUuid, zPJ, zUuid);
       @ <tr><th>Downloads:</th><td>
       @ %z(href("%s",zUrl))Tarball</a>
-      @ | %z(href("%R/zip/%t-%S.zip?r=%!S",zPJ,zUuid,zUuid))ZIP archive</a>
-      @ | %z(href("%R/sqlar/%t-%S.sqlar?r=%!S",zPJ,zUuid,zUuid))\
+      @ | %z(href("%R/zip/%S/%t-%S.zip",zUuid, zPJ,zUuid))ZIP archive</a>
+      @ | %z(href("%R/sqlar/%S/%t-%S.sqlar",zUuid,zPJ,zUuid))\
       @ SQL archive</a></td></tr>
       fossil_free(zUrl);
       blob_reset(&projName);
@@ -937,10 +937,7 @@ void winfo_page(void){
   if( g.perm.Setup ){
     @ (%d(rid))
   }
-  modPending = moderation_pending(rid);
-  if( modPending ){
-    @ <span class="modpending">*** Awaiting Moderator Approval ***</span>
-  }
+  modPending = moderation_pending_www(rid);
   @ </td></tr>
   @ <tr><th>Page&nbsp;Name:</th><td>%h(pWiki->zWikiTitle)</td></tr>
   @ <tr><th>Date:</th><td>
@@ -983,21 +980,6 @@ void winfo_page(void){
   style_footer();
 }
 
-/*
-** Show a webpage error message
-*/
-void webpage_error(const char *zFormat, ...){
-  va_list ap;
-  const char *z;
-  va_start(ap, zFormat);
-  z = vmprintf(zFormat, ap);
-  va_end(ap);
-  style_header("URL Error");
-  @ <h1>Error</h1>
-  @ <p>%h(z)</p>
-  style_footer();
-}
-
 /*
 ** Find an check-in based on query parameter zParam and parse its
 ** manifest.  Return the number of errors.
@@ -1248,6 +1230,7 @@ void vdiff_page(void){
 #define OBJTYPE_TAG        0x0040
 #define OBJTYPE_SYMLINK    0x0080
 #define OBJTYPE_EXE        0x0100
+#define OBJTYPE_FORUM      0x0200
 
 /*
 ** Possible flags for the second parameter to
@@ -1440,6 +1423,9 @@ int object_description(
         }else{
           @ Attachment to technote
         }
+      }else if( zType[0]=='f' ){
+        objType |= OBJTYPE_FORUM;
+        @ Forum post
       }else{
         @ Tag referencing
       }
@@ -1466,7 +1452,6 @@ int object_description(
   );
   while( db_step(&q)==SQLITE_ROW ){
     const char *zTarget = db_column_text(&q, 0);
-    int nTarget = db_column_bytes(&q, 0);
     const char *zFilename = db_column_text(&q, 1);
     const char *zDate = db_column_text(&q, 2);
     const char *zUser = db_column_text(&q, 3);
@@ -1477,7 +1462,7 @@ int object_description(
       @ Attachment "%h(zFilename)" to
     }
     objType |= OBJTYPE_ATTACHMENT;
-    if( nTarget==UUID_SIZE && validate16(zTarget,UUID_SIZE) ){
+    if( fossil_is_uuid(zTarget) ){
       if ( db_exists("SELECT 1 FROM tag WHERE tagname='tkt-%q'",
             zTarget)
       ){
@@ -1910,7 +1895,7 @@ void output_text_with_line_numbers(
       iStart = iEnd = atoi(&zLn[i++]);
     }while( zLn[i] && iStart && iEnd );
   }
-  db_prepare(&q, "SELECT min(iStart), iEnd FROM lnos");
+  db_prepare(&q, "SELECT min(iStart), max(iEnd) FROM lnos");
   if( db_step(&q)==SQLITE_ROW ){
     iStart = db_column_int(&q, 0);
     iEnd = db_column_int(&q, 1);
@@ -2187,7 +2172,7 @@ void tinfo_page(void){
   int rid;
   char *zDate;
   const char *zUuid;
-  char zTktName[UUID_SIZE+1];
+  char zTktName[HNAME_MAX+1];
   Manifest *pTktChng;
   int modPending;
   const char *zModAction;
@@ -2208,7 +2193,7 @@ void tinfo_page(void){
   pTktChng = manifest_get(rid, CFTYPE_TICKET, 0);
   if( pTktChng==0 ) fossil_redirect_home();
   zDate = db_text(0, "SELECT datetime(%.12f)", pTktChng->rDate);
-  memcpy(zTktName, pTktChng->zTicketUuid, UUID_SIZE+1);
+  sqlite3_snprintf(sizeof(zTktName), zTktName, "%s", pTktChng->zTicketUuid);
   if( g.perm.ModTkt && (zModAction = P("modaction"))!=0 ){
     if( strcmp(zModAction,"delete")==0 ){
       moderation_disapprove(rid);
@@ -2251,10 +2236,7 @@ void tinfo_page(void){
   if( g.perm.Setup ){
     @ (%d(rid))
   }
-  modPending = moderation_pending(rid);
-  if( modPending ){
-    @ <span class="modpending">*** Awaiting Moderator Approval ***</span>
-  }
+  modPending = moderation_pending_www(rid);
   @ <tr><th>Ticket:</th>
   @ <td>%z(href("%R/tktview/%s",zTktName))%s(zTktName)</a>
   if( zTktTitle ){
@@ -2368,6 +2350,11 @@ void info_page(void){
   if( db_exists("SELECT 1 FROM attachment WHERE attachid=%d", rid) ){
     ainfo_page();
   }else
+  if( db_table_exists("repository","forumpost")
+   && db_exists("SELECT 1 FROM forumpost WHERE fpid=%d", rid)
+  ){
+    forumthread_page();
+  }else
   {
     artifact_page();
   }
@@ -2620,7 +2607,7 @@ void ci_edit_page(void){
   zNewBranch = PDT("brname","");
   zCloseFlag = P("close") ? " checked" : "";
   zHideFlag = P("hide") ? " checked" : "";
-  if( P("apply") ){
+  if( P("apply") && cgi_csrf_safe(1) ){
     Blob ctrl;
     char *zNow;
 
