@@ -751,6 +751,7 @@ static int searchableCmd(
 ** "markdown"        = FOSSIL_ENABLE_MARKDOWN
 ** "unicodeCmdLine"  = !BROKEN_MINGW_CMDLINE
 ** "dynamicBuild"    = FOSSIL_DYNAMIC_BUILD
+** "mman"            = USE_MMAN_H
 ** "see"             = USE_SEE
 **
 ** Specifying an unknown feature will return a value of false, it will not
@@ -832,6 +833,11 @@ static int hasfeatureCmd(
     rc = 1;
   }
 #endif
+#if defined(USE_MMAN_H)
+  else if( 0 == fossil_strnicmp( zArg, "mman\0", 5 ) ){
+    rc = 1;
+  }
+#endif
 #if defined(USE_SEE)
   else if( 0 == fossil_strnicmp( zArg, "see\0", 4 ) ){
     rc = 1;
@@ -1250,7 +1256,7 @@ static int renderCmd(
 /*
 ** TH1 command: styleHeader TITLE
 **
-** Render the configured style header.
+** Render the configured style header for the selected skin.
 */
 static int styleHeaderCmd(
   Th_Interp *interp,
@@ -1275,7 +1281,7 @@ static int styleHeaderCmd(
 /*
 ** TH1 command: styleFooter
 **
-** Render the configured style footer.
+** Render the configured style footer for the selected skin.
 */
 static int styleFooterCmd(
   Th_Interp *interp,
@@ -1297,6 +1303,34 @@ static int styleFooterCmd(
   }
 }
 
+/*
+** TH1 command: styleScript
+**
+** Render the configured JavaScript for the selected skin.
+*/
+static int styleScriptCmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  if( argc!=1 ){
+    return Th_WrongNumArgs(interp, "styleScript");
+  }
+  if( Th_IsRepositoryOpen() ){
+    const char *zScript = skin_get("js");
+    if( zScript==0 ) zScript = "";
+    Th_Render(zScript);
+    Th_SetResult(interp, 0, 0);
+    return TH_OK;
+  }else{
+    Th_SetResult(interp, "repository unavailable", -1);
+    return TH_ERROR;
+  }
+}
+
+
 /*
 ** TH1 command: artifact ID ?FILENAME?
 **
@@ -1980,8 +2014,9 @@ void Th_FossilInit(u32 flags){
     {"searchable",    searchableCmd,        0},
     {"setParameter",  setParameterCmd,      0},
     {"setting",       settingCmd,           0},
-    {"styleHeader",   styleHeaderCmd,       0},
     {"styleFooter",   styleFooterCmd,       0},
+    {"styleHeader",   styleHeaderCmd,       0},
+    {"styleScript",   styleScriptCmd,       0},
     {"tclReady",      tclReadyCmd,          0},
     {"trace",         traceCmd,             0},
     {"stime",         stimeCmd,             0},
